import xbmc
import xbmcgui
from database import dbio
from dialogs import context
from emby import listitem
from core import common
from . import utils, pluginmenu

SelectOptions = {'Refresh': utils.Translate(30410), 'Delete': utils.Translate(30409), 'Addon': utils.Translate(30408), 'AddFav': utils.Translate(30405), 'RemoveFav': utils.Translate(30406), 'SpecialFeatures': utils.Translate(33231), "Watch together": "Watch together"}


def load_item():
    item = None
    ServerId = xbmc.getInfoLabel('ListItem.Property(embyserverid)')
    emby_id = xbmc.getInfoLabel('ListItem.Property(embyid)')

    if not ServerId:
        for ServerId in utils.EmbyServers:
            kodi_id = xbmc.getInfoLabel('ListItem.DBID')
            media = xbmc.getInfoLabel('ListItem.DBTYPE')
            embydb = dbio.DBOpenRO(ServerId, "load_item")
            item = embydb.get_item_by_KodiId_KodiType(kodi_id, media)
            dbio.DBCloseRO(ServerId, "load_item")

            if item:
                item = item[0]
                break

        return item, ServerId

    pluginmenu.reset_querycache() # Clear Cache
    return (emby_id,), ServerId

def delete_item(LoadItem, item=None, ServerId=""):  # threaded by caller
    if utils.Dialog.yesno(heading=utils.addon_name, message=utils.Translate(33015)):
        if LoadItem:
            item, ServerId = load_item()

        if item:
            utils.EmbyServers[ServerId].API.delete_item(item[0])
            utils.EmbyServers[ServerId].library.removed([item[0]])
            xbmc.executebuiltin("Container.Refresh()")

def specials():
    SpecialFeaturesSelections = []
    item, ServerId = load_item()

    if not item:
        return

    # Load SpecialFeatures
    embydb = dbio.DBOpenRO(ServerId, "specials")
    SpecialFeaturesIds = embydb.get_special_features(item[0])

    for SpecialFeaturesId in SpecialFeaturesIds:
        SpecialFeaturesMediasources = embydb.get_mediasource(SpecialFeaturesId[0])
        SpecialFeaturesSelections.append({"Name": SpecialFeaturesMediasources[0][4], "Id": SpecialFeaturesId[0]})

    dbio.DBCloseRO(ServerId, "specials")
    MenuData = []

    for SpecialFeaturesSelection in SpecialFeaturesSelections:
        MenuData.append(SpecialFeaturesSelection['Name'])

    resp = utils.Dialog.select(utils.Translate(33231), MenuData)

    if resp < 0:
        return

    ItemData = SpecialFeaturesSelections[resp]
    SpecialFeatureItem = utils.EmbyServers[ServerId].API.get_Item(ItemData['Id'], ['Movie'], True, False, True, True)

    if SpecialFeatureItem:
        li = listitem.set_ListItem(SpecialFeatureItem, ServerId)
        path, _ = common.get_path_type_from_item(ServerId, SpecialFeatureItem)
        li.setProperty('path', path)
        playlist = xbmc.PlayList(xbmc.PLAYLIST_VIDEO)
        Pos = playlist.getposition() + 1
        playlist.add(path, li, index=Pos)
        utils.XbmcPlayer.play(playlist, li, False, Pos)

def select_menu(ListItemLabel):
    options = []
    item, ServerId = load_item()

    if not item:
        return

    if len(item) > 9:
        if item[10]:
            options.append(SelectOptions['RemoveFav'])
        else:
            options.append(SelectOptions['AddFav'])

    options.append(SelectOptions['Refresh'])

    if utils.enableContextDelete:
        options.append(SelectOptions['Delete'])

    if item[2] in ('Episode', 'Movie', 'MusicVideo', 'Audio'):
        options.append(SelectOptions['Watch together'])

    options.append(SelectOptions['Addon'])
    context_menu = context.ContextMenu("script-emby-context.xml", *utils.CustomDialogParameters)
    context_menu.PassVar(options)
    context_menu.doModal()

    if context_menu.is_selected():
        selected_option = context_menu.get_selected()

        if selected_option == SelectOptions['Refresh']:
            utils.EmbyServers[ServerId].API.refresh_item(item[0])
        elif selected_option == SelectOptions['AddFav']:
            utils.EmbyServers[ServerId].API.favorite(item[0], True)
        elif selected_option == SelectOptions['RemoveFav']:
            utils.EmbyServers[ServerId].API.favorite(item[0], False)
        elif selected_option == SelectOptions['Addon']:
            xbmc.executebuiltin(f"Addon.OpenSettings({utils.PluginId})")
        elif selected_option == SelectOptions['Delete']:
            delete_item(False, item, ServerId)
        elif selected_option == SelectOptions['Watch together']:
            ActiveSessions = utils.EmbyServers[ServerId].API.get_active_sessions()
            ActiveUsers = []
            ActiveUsersSessions = []

            for ActiveSession in ActiveSessions:
                if ActiveSession['SupportsRemoteControl'] and ActiveSession['Id'] != utils.EmbyServers[ServerId].EmbySession[0]['Id']:
                    ActiveUsers.append(f"{ActiveSession['DeviceName']}, {ActiveSession['UserName']}")
                    ActiveUsersSessions.append(ActiveSession['Id'])

            Selections = utils.Dialog.multiselect("Select users", ActiveUsers)

            if not Selections:
                return

            ProgressBar = xbmcgui.DialogProgress()
            ProgressBar.create("Watch together")
            utils.WatchTogetherParticipants = [utils.EmbyServers[ServerId].EmbySession[0]['Id']]

            for Selection in Selections:
                utils.EmbyServers[ServerId].API.send_text_msg(ActiveUsersSessions[Selection], "watch together request", f"{utils.EmbyServers[ServerId].EmbySession[0]['Id']};{ListItemLabel};60")

            # wait 60 seconds or all accepted
            UsersSessionsSet = []

            for Index in range(60):
                if utils.sleep(1): # wait here for user confirmation or timeout
                    return

                ProgressBar.update(int(Index / 0.6), "Waiting for particitians")

                for SessionId in utils.WatchTogetherParticipants:
                    if not SessionId in UsersSessionsSet:
                        UsersSessionsSet.append(SessionId)
                        # Make all clients stream synchron
                        utils.EmbyServers[ServerId].API.send_play(SessionId, item[0], "PlayNow", 0)
                        utils.EmbyServers[ServerId].API.send_pause(SessionId)
                        utils.EmbyServers[ServerId].API.send_seek(SessionId, 0)

                if len(Selections) + 1 == len(utils.WatchTogetherParticipants):
                    break

             # give time to prepare streams for all client devices
            ProgressBar.update(0, "Starting...")

            for Index in range(10):
                if utils.sleep(1): # wait here for user confirmation or timeout
                    return

                ProgressBar.update(int(Index / 0.1), "Starting...")

            ProgressBar.close()
            utils.WatchTogetherInProgress = True

            for SessionId in utils.WatchTogetherParticipants:
                utils.EmbyServers[ServerId].API.send_unpause(SessionId)

def Record():
    Temp = xbmc.getInfoLabel('ListItem.EPGEventIcon') # Icon path has Emby's EPG programId assinged (workaround)
    Temp = Temp[Temp.find("@") + 1:].replace("/","")
    Temp = Temp.split("Z")
    Timers = utils.EmbyServers[Temp[0]].API.get_timer(Temp[1])
    TimerId = 0

    if Timers:
        for Timer in Timers:
            if Timer['ProgramId'] == Temp[1]:
                TimerId = Timer['ProgramInfo']['TimerId']
                break

    if TimerId: # Delete recording
        if utils.Dialog.yesno(heading=utils.addon_name, message="Delete timer"):
            utils.EmbyServers[Temp[0]].API.delete_timer(TimerId)
    else: # Add recoding
        if utils.Dialog.yesno(heading=utils.addon_name, message="Add timer"):
            utils.EmbyServers[Temp[0]].API.set_timer(Temp[1])
